<?php
/**
 * Plugin Name: Fullscreen Ajax Loader
 * Description: Show full width and height animation with overlay over each ajax request
 * Version: 1.2
 * Author: From Poland With Dev
 * Author URI: https://frompolandwithdev.com
 * License: GPL3
 * Text Domain: fullscreen-ajax-loader
 */


namespace FPWD;

if ( ! defined( 'FS_LOADER_VER' ) ) {
	define( 'FS_LOADER_VER', '1.2' );
}

class Fullscreen_Ajax_Loader {

	public function __construct() {

	}

	public function add_hooks() {

		add_action( 'plugins_loaded', [
			$this,
			'load_textdomain',
		] );

		add_action( 'customize_register', [
			$this,
			'register_customizer',
		] );

		add_action( 'wp_head', [
			$this,
			'inline_style',
		], 99 );
		add_action( 'wp_footer', [
			$this,
			'print_html',
		] );
		add_action( 'wp_enqueue_scripts', [
			$this,
			'enqueue_style',
		] );
		add_action( 'wp_enqueue_scripts', [
			$this,
			'enqueue_scripts',
		] );

		add_action( 'wp_enqueue_admin_scripts', [
			$this,
			'enqueue_admin_scripts',
		] );

		add_action( 'wp_ajax_demo_loading', [
			$this,
			'demo_loading',
		] );

		add_filter( 'plugin_action_links', [
			$this,
			'plugin_action_links',
		], 10, 2 );
	}

	/**
	 * @param array $links
	 * @param string $file
	 *
	 * @return array
	 */
	public function plugin_action_links( $links, $file ) {
		if ( $file !== plugin_basename( __FILE__ ) ) {
			return $links;
		}

		$panel_link        = add_query_arg( [ 'autofocus[section]' => 'fs_loader_settings' ], admin_url( 'customize.php' ) );
		$links['settings'] = sprintf( "<a href=\"%s\">%s</a>", $panel_link, __( 'Settings', 'fullscreen-ajax-loader' ) );

		return $links;
	}

	function load_textdomain() {
		load_plugin_textdomain( 'fullscreen-ajax-loader', false, basename( dirname( __FILE__ ) ) . '/languages' );
	}

	public function register_customizer( $wp_customize ) {
		$wp_customize->add_section( 'fs_loader_settings', [
			'title'    => __( 'Fullscreen Loader', 'fullscreen-ajax-loader' ),
			'priority' => 999,
		] );

		$wp_customize->add_setting( 'fs_template', array(
			'type'      => 'option',
			'default'   => 'circular.php',
			'transport' => 'refresh',
		) );

		$wp_customize->add_control( 'fs_template', array(
			'label'    => __( 'Active Template', 'fullscreen-ajax-loader' ),
			'section'  => 'fs_loader_settings',
			'settings' => 'fs_template',
			'type'     => 'select',
			'choices'  => [
				'circular.php'         => 'Circular',
				'floating-bars.php'    => 'Floating Bars',
				'floating-circles.php' => 'Floating Circles',
				'load-ball.php'        => 'Load Ball',
				'spiral.php'           => 'Spiral',
				'custom.php'           => 'Custom',
			],
		) );

		$wp_customize->add_setting( 'fs_loader_img', array(
			'type'      => 'option',
			'default'   => '',
			'transport' => 'refresh',
		) );

		$wp_customize->add_control( new \WP_Customize_Cropped_Image_Control( $wp_customize, 'fs_loader_img', array(
			'label'    => __( 'Upload custom loader image (this setting overrides active theme)', 'fullscreen-ajax-loader' ),
			'section'  => 'fs_loader_settings',
			'settings' => 'fs_loader_img',
			'width'    => 400,
			'height'   => 400,
		) ) );

		$wp_customize->add_setting( 'fs_background_color', array(
			'type'      => 'option',
			'default'   => '#43C6E4',
			'transport' => 'refresh',
		) );

		$wp_customize->add_control( new \WP_Customize_Color_Control( $wp_customize, 'fs_background_color', array(
			'label'    => __( 'Background Color', 'fullscreen-ajax-loader' ),
			'section'  => 'fs_loader_settings',
			'settings' => 'fs_background_color',
		) ) );

		$wp_customize->add_setting( 'fs_background_opacity', array(
			'type'      => 'option',
			'default'   => '0.8',
			'transport' => 'refresh',
		) );

		$wp_customize->add_control( 'fs_background_opacity', array(
			'label'    => __( 'Background Opacity', 'fullscreen-ajax-loader' ),
			'section'  => 'fs_loader_settings',
			'settings' => 'fs_background_opacity',
			'type'     => 'input',
		) );

		$wp_customize->add_setting( 'fs_loader_scale', array(
			'type'      => 'option',
			'default'   => '1',
			'transport' => 'refresh',
		) );

		$wp_customize->add_control( 'fs_loader_scale', array(
			'label'    => __( 'Scale loader size', 'fullscreen-ajax-loader' ),
			'section'  => 'fs_loader_settings',
			'settings' => 'fs_loader_scale',
			'type'     => 'input',
		) );

		$wp_customize->add_setting( 'fs_wc_compat', array(
			'type'      => 'option',
			'default'   => '1',
			'transport' => 'refresh',
		) );

		$wp_customize->add_control( 'fs_wc_compat', array(
			'label'    => __( 'WooCommerce Compatibility (disable default WC loader)', 'fullscreen-ajax-loader' ),
			'section'  => 'fs_loader_settings',
			'settings' => 'fs_wc_compat',
			'type'     => 'checkbox',
		) );

		$wp_customize->add_setting( 'fs_exclude_urls', array(
			'type'      => 'option',
			'default'   => '',
			'transport' => 'refresh',
		) );

		$wp_customize->add_control( 'fs_exclude_urls', array(
			'label'    => __( 'Exclude URLs (slugs list, each url in new line)', 'fullscreen-ajax-loader' ),
			'section'  => 'fs_loader_settings',
			'settings' => 'fs_exclude_urls',
			'type'     => 'textarea',
		) );

		$wp_customize->add_setting( 'fs_exclude_params', array(
			'type'      => 'option',
			'default'   => '',
			'transport' => 'refresh',
		) );

		$wp_customize->add_control( 'fs_exclude_params', array(
			'label'    => __( 'Exclude request parameters (if param is included to request, loader will be not displayed, each param in new line)', 'fullscreen-ajax-loader' ),
			'section'  => 'fs_loader_settings',
			'settings' => 'fs_exclude_params',
			'type'     => 'textarea',
		) );

	}

	public function enqueue_style() {
		wp_enqueue_style( 'fs-ajax-loader', plugin_dir_url( __FILE__ ) . 'assets/css/fs-ajax-loader.css', null, FS_LOADER_VER );
	}

	public function inline_style() {
		$bg_color = get_option( 'fs_background_color', '#ffffff' );
		list( $r, $g, $b ) = sscanf( $bg_color, "#%02x%02x%02x" );
		$bg_opacity = get_option( 'fs_background_opacity', '0.8' );
		$scale      = get_option( 'fs_loader_scale', '1' );
		$wc_compat  = get_option( 'fs_wc_compat' );

		?>
		<style>
			#fpwd-fullscreen-ajax-loader {
				background: rgba(<?php echo $r; ?>, <?php echo $g; ?>, <?php echo $b; ?>, <?php echo $bg_opacity; ?>) !important;
			}

			#fpwd-fullscreen-ajax-loader > .loader-wrapper {
				transform: scale(<?php echo $scale; ?>);
			}

			<?php if(!empty($wc_compat)): ?>
			.blockOverlay {
				display: none !important;
				position: fixed !important;
				width: 0 !important;
				height: 0 !important;
				left: -1000em !important;
				top: -1000em !important;
				visibility: hidden !important;
			}

			<?php endif; ?>
		</style>
		<?php
	}

	public function print_html() {
		?>
		<div id="fpwd-fullscreen-ajax-loader" class="loader-hidden">
			<div class="loader-wrapper">
				<?php
				$img_id = get_option('fs_loader_img');
				if(!empty($img_id)) {
					$img = wp_get_attachment_image( $img_id, '99999x99999' );
					echo $img;
				}
				else {
					$active_template = get_option( 'fs_template', 'circular.php' );
					$this->get_template( $active_template );
				}
				?>
			</div>
		</div>
		<?php
	}

	public function enqueue_scripts() {
		global $wp;
		$exclude_urls = explode( PHP_EOL, get_option( 'fs_exclude_urls' ) );
		$exclude_urls = array_map( function( $val ) {
			return ltrim( $val, '/' );
		}, $exclude_urls );

		$current_slug = $wp->request;

		if ( ! in_array( $current_slug, $exclude_urls ) ) {
			wp_enqueue_script( 'fs-ajax-loader', plugin_dir_url( __FILE__ ) . 'assets/js/fs-ajax-loader.js', null, FS_LOADER_VER, true );
			$exclude_params = array_map( 'trim', explode( PHP_EOL, get_option( 'fs_exclude_params' ) ) );
			wp_localize_script( 'fs-ajax-loader', 'FS',
				[
					'ajaxUrl'        => admin_url( 'admin-ajax.php' ),
					'excludedParams' => rtrim( implode( '|', $exclude_params ), '|' ),
				]
			);
		}
	}

	public function enqueue_admin_scripts() {
		wp_enqueue_script( 'fs-ajax-loader', plugin_dir_url( __FILE__ ) . 'assets/js/fs-ajax-loader.js', null, FS_LOADER_VER, true );
		wp_localize_script( 'fs-ajax-loader', 'FS', [ 'ajaxUrl' => admin_url( 'admin-ajax.php' ) ] );
	}

	public function get_template( $template_name, $args = array(), $template_path = '', $default_path = '' ) {
		if ( ! empty( $args ) && is_array( $args ) ) {
			extract( $args ); // @codingStandardsIgnoreLine
		}

		$located = $this->locate_template( $template_name, $template_path, $default_path );

		if ( ! file_exists( $located ) ) {
			/* translators: %s template */
			_doing_it_wrong( __FUNCTION__, sprintf( __( '%s does not exist.', 'fullscreen-ajax-loader' ), '<code>' . $located . '</code>' ), '2.1' );

			return;
		}

		// Allow 3rd party plugin filter template file from their plugin.
		$located = apply_filters( 'fsal_get_template', $located, $template_name, $args, $template_path, $default_path );

		include $located;


	}

	public function locate_template( $template_name, $template_path = '', $default_path = '' ) {
		if ( ! $template_path ) {
			$template_path = apply_filters( 'fsal_template_path', 'fullscreen-ajax-loader/' );;
		}

		if ( ! $default_path ) {
			$default_path = $this->plugin_path() . '/templates/';
		}

		// Look within passed path within the theme - this is priority.
		$template = locate_template(
			array(
				trailingslashit( $template_path ) . $template_name,
				$template_name,
			)
		);

		// Get default template/.
		if ( ! $template ) {
			$template = $default_path . $template_name;
		}

		// Return what we found.
		return apply_filters( 'fsal_locate_template', $template, $template_name, $template_path );
	}

	private function plugin_path() {
		return untrailingslashit( plugin_dir_path( __FILE__ ) );
	}

	public function demo_loading() {
		sleep( 5 );
		wp_die( 'SUCCESS' );
	}
}

( new Fullscreen_Ajax_Loader() )->add_hooks();
